#include <linux/module.h>
#include <linux/proc_fs.h>
#include <asm/io.h>

/**
 *	csc_write: Write a value to a Chip Setup and Control Register
 *	These registers hide behind i/o ports 0x22/0x23
 */
void csc_write(u8 port, u8 value)
{
	outb(port, 0x22);
	outb(value,0x23);
}


/**
 *	csc_read: Read a value from a Chip Setup and Control Register
 */
u8 csc_read(u8 port)
{
	outb(port,0x22);
	return inb(0x23);
}


/**
 *	netdriver_read_proc: read the network configuration switches 
 * 	every time /proc/incotec/network_switch was read
 */
int netdriver_read_proc(char *buf, char **start, off_t offset, 
		int count, int *eof, void *data)
{
	u8 network_switch; 

	network_switch = inb(0x210);
	return(sprintf(buf, "%02x %i\n", network_switch, network_switch));
}


/**
 *	init_module: called when module is loaded
 */

int init_module(void)
{
	printk("<1>Incotec Network Driver v0.1.1 initializing.\n");

	proc_mkdir("incotec",NULL);	
	create_proc_read_entry(
		"incotec/network_switch",
		0,					/* default mode     */
		NULL,					/* parent dir       */
		netdriver_read_proc,			/* handler function */
		NULL					/* client data      */
	);

	/* Initialize CS1: base=0x210 -> 0x210-0x21F */
	csc_write(0xB6, 0x10);				/* CS1 I/O low-byte */
	csc_write(0xB7,(csc_read(0xB7) & 0xC0) | 0x02);	/* CS1 I/O high byte and mask 		*/
	csc_write(0xB8,csc_read(0xB8) & 0x88);		/* CS0/CS1 not add.  qualified, 8 Bit 	*/
	csc_write(0xB2,0x10);				/* map GPIO_CSA=CS0 */ 
							/* map GPIO_CSB=CS1 */
	csc_write(0xA0, csc_read(0xA0) | 0x05);		/* CS0/1 is output  */

	return 0;
}


/** 
 *	cleanup_module: called when module is removed
 */
void cleanup_module(void)
{
	printk("<1>Incotec Network Driver v0.1.1 exiting.\n");
	remove_proc_entry("incotec/network_switch",NULL);
	remove_proc_entry("incotec",NULL);	
}
